/*******************************************************************************************************
  lora Programs for Arduino - Copyright of the author Stuart Robinson - 15/01/20
  This program is supplied as is, it is up to the user of the program to decide if the program is
  suitable for the intended purpose and free from errors.
*******************************************************************************************************/

// *******  Setup hardware pin definitions here ! *******
//These are the pin definitions for one of the LoRaTracker boards, the ESP32_Micro_Node, be sure to change
//them to match your own setup. You will also need to connect up the pins for the SPI bus, which on the 
//ESP32_Micro_Node are SCK on pin 13, MISO on pin 19 and MOSI on pin 23. Some pins such as DIO1, DIO2 and
//BUZZER may not be in used by this sketch so they do not need to be connected and should be set to -1.

#define NSS 5                                   //M5 select pin on LoRa device
#define SCK 18                                  //M5 SCK on SPI3
#define MISO 19                                 //M5 MISO on SPI3 
#define MOSI 23                                 //M5 MOSI on SPI3 
#define NRESET 26                               //M5 reset pin on LoRa device
#define LED1 -1                                 //on board LED, high for on
#define DIO0 36                                 //M5 DIO0 pin on LoRa device, used for RX and TX done 
#define DIO1 -1                                 //DIO1 pin on LoRa device, normally not used so set to -1 
#define DIO2 -1                                 //DIO2 pin on LoRa device, normally not used so set to -1
#define BUZZER -1                               //pin for buzzer, set to -1 if not used 
#define VCCPOWER -1                             //pin controls power to external devices

#define LORA_DEVICE DEVICE_SX1278               //we need to define the device we are using
#define RXBUFFER_SIZE 108                       //RX buffer size 

//                    *******  Setup LoRa Parameters Here ! *******
const uint32_t Frequency      = 433775000;   //frequency of transmissions in hertz
const uint32_t Offset         = 0;           //offset frequency for calibration purposes
const uint8_t Bandwidth       = LORA_BW_125; //LoRa bandwidth
const uint8_t SpreadingFactor = LORA_SF12;   //LoRa spreading factor
const uint8_t CodeRate        = LORA_CR_4_5; //LoRa coding rate
const uint8_t Optimisation    = LDRO_AUTO;   //low data rate optimisation setting, normally set to auto
const int8_t TXpower          = 10;          //LoRa transmit power in dBm
const uint16_t packet_delay   = 1000;        //mS delay between packets
