// Version mit 4.Fenster fuer Rohdaten
// NEU: Anzeige ankommender Werte von Versorgungsspannung in Volt
// Version mit DST-, BRG ohne Sprachausgabe

// BT name: "M5S_2_LoRa_APRS"
// NEW: version using pusbuttons for DST/BRG "ON/OFF" 

// LoRa-APRS-Decoder for use with M5Stack hardware
// with decoding compressed (BASE[91])and 
// uncompressed (APRS) protocols; sending Standard
// APRS data via Bluetooth

// program by dj7oo@t-online.de with using partly source and 
// LoRa Library by Stuart Robinson, GW7HPW: 
// https://github.com/StuartsProjects/SX12XX-LoRa 

/********************************************************************
  lora Programs for Arduino - Copyright of the author Stuart Robinson 
  - 20/02/20
  This program is supplied as is, it is up to the user of the program
  to decide if the program is suitable for the intended purpose and 
  free from errors.
*********************************************************************/

#define Program_Version "V1.1"

#include <SPI.h>                                 //the SX127X device is SPI based so load the SPI library
#include <SX127XLT.h>                            //include the appropriate library   
#include "Settings.h"                            //include the setiings file, frequencies, LoRa settings etc   
#include <M5Stack.h>
#include "BluetoothSerial.h"
#include <Arduino.h>

HardwareSerial mySoftwareSerial(1);

SX127XLT LT;                                     //create a library class instance called LT
BluetoothSerial bluetooth;

uint32_t RXpacketCount;
uint32_t errors;
uint8_t RXBUFFER[RXBUFFER_SIZE];                 //create the buffer that received packets are copied into
uint8_t RXPacketL;                               //stores length of packet received
int8_t  PacketRSSI;                              //stores RSSI of received packet
int8_t  PacketSNR;                               //stores signal to noise ratio of received packet
String Inpacket, Inpacket2, outString;
char tab, symb;
String Alt, Ns, Ew, Tcall, addtext, aprsdata;
float Tlat,Tlon,Talt,Lat,Lon;
byte start_with = 0;
byte end_with = 78;    
byte CRC = 0;
bool APRS_Mode;

int posvolt;
String battvolt;

String temp, humi,pres;
float  tempc, altic, presc;
int cnt1wx, cnt2wx, cnt3wx;

String xLat = "";
String Ns2 = "";
String xLon = "";
String Ew2 = "";
String APRS = "";
String humi2, addtext2;

float dLat, Lat2c, Lat1c, Distance, Distance2;
float dLon, Lon2c, Lon1c, Bearing, Bearing2;
unsigned long num;

float Lat1a = 49.96;   // Lat Eigen Fixposition
float Lon1a = 8.1993;  // Lon Eigen Fixposition 

#define TONE_FREQ 900
#define TONE_STOP 0
#define TONE_PLAY 1

const int button1 = 39;  
const int button2 = 38;  
const int button3 = 37;

volatile int State1 = 0;         
volatile int State2 = 0;         
volatile int State3 = 0;         

static unsigned char gucVol = 6;
static unsigned char gucToneSts = 0;

bool mrk_x = 1;
bool mrk_y = 1;
int  mrk_cnt = 0;
int  cnt = 0;

#define Deg2Rad 0.0174532925 // value for calc. radians

/////////////////////////////////////////////////////////////////////////////
void loop()
{
//  Serial.println("LOOP");
  RXPacketL = LT.receive(RXBUFFER, RXBUFFER_SIZE, 60000, WAIT_RX); 
  digitalWrite(LED1, HIGH);                      //something has happened
  if (BUZZER > 0)                                //turn buzzer on
  {
  digitalWrite(BUZZER, HIGH);
  }
  PacketRSSI = LT.readPacketRSSI();              //read the recived RSSI value
  PacketSNR = LT.readPacketSNR();                //read the received SNR value
  if (RXPacketL == 0)                            //if the LT.receive() function detects an error, RXpacketL == 0
  {
  packet_is_Error();
  }
  else
  {
  packet_is_OK();
  }
  if (BUZZER > 0)
  {
  digitalWrite(BUZZER, LOW);                    //buzzer off
  }
  digitalWrite(LED1, LOW);                        //LED off
//  Serial.println("*");
}

////////////////////////////////////////////////////
void Calc_Bearing()      // calculate bearings
{
  dLon = Lon2c-Lon1c;
  Bearing = atan2(sin(dLon)*cos(Lat2c),cos(Lat1c)*sin(Lat2c)-sin(Lat1c)*cos(Lat2c)*cos(dLon));
  Bearing=Bearing*57.29578;
  if(Bearing<0) Bearing+=360;  
}

///////////////////////////////////////////////////
void Calc_Distance()     // calculate distances
{
  dLat = Lat2c-Lat1c;                    
  float a = sin(dLat/2) * sin(dLat/2) + sin(dLon/2) * sin(dLon/2) * cos(Lat1c) * cos(Lat2c); 
  float c = 2 * atan2(sqrt(a), sqrt(1-a)); 
// Distance = 3959 * c;                // activate for miles
   Distance = 6371 * c;                // activate for kilometers
}

/////////////////////////////////////////////////////////
void packet_is_Error()
{
//  Serial.println("NOK");
  uint16_t IRQStatus;
  IRQStatus = LT.readIrqStatus();                   //read the LoRa device IRQ status register
  printElapsedTime();                               //print elapsed time to Serial Monitor
  if (IRQStatus & IRQ_RX_TIMEOUT)                   //check for an RX timeout
  {
//    Serial.println(F(" RXTimeout"));
  }
  else
  {
    errors++;
    Serial.print(F(" PacketError"));
    Serial.print(F(",RSSI,"));
    Serial.print(PacketRSSI);
    Serial.print(F("dBm,SNR,"));
    Serial.print(PacketSNR);
    Serial.print(F("dB,Length,"));
    Serial.print(LT.readRXPacketL());               //get the real packet length
    Serial.print(F(",Packets,"));
    Serial.print(RXpacketCount);
    Serial.print(F(",Errors,"));
    Serial.print(errors);
    Serial.print(F(",IRQreg,"));
    Serial.print(IRQStatus, HEX);
    LT.printIrqStatus();                            //print the names of the IRQ registers set
  }
  delay(250);                                       //gives a longer buzzer and LED flash for error
}

////////////////////////////////////////////////////////////////
void printElapsedTime()
{
  float seconds;
  seconds = millis() / 1000;
//  Serial.print(seconds, 0);
//  Serial.print(F("s"));
}

/////////////////////////////////////////////////////////
static void SetVolume( unsigned char ucVol )
{
    ledcWrite( TONE_PIN_CHANNEL, 0x1FF>>(9-ucVol) ); 
    return;
}

///////////////////////////////////////////////
void build_aprs()
  {
// building APRS standard sentence
 outString = (Tcall);
 outString += ">APRS:!";
 if(Tlat<10) {outString += "0"; }
 outString += String(Lat,2);    
 outString += Ns;    
 outString += tab;   // APRS table
 if(Tlon<100) {outString += "0"; }
 if(Tlon<10) {outString += "0"; }
 outString += String(Lon,2); 
 outString += Ew; 
 outString += symb;  // APRS symbol
 outString += (" ");
  
// outString += " /A=";    
// if(Talt<10000) { outString += "0"; }
// if(Talt<1000)  { outString += "0"; }
// if(Talt<100)   { outString += "0"; }
// if(Talt<10)    { outString += "0"; }
// outString += String(Talt,0);    
// outString += "m ";

 outString += (addtext); 
 outString += (" RSSI:"); 
 outString += (PacketRSSI);
 outString += " SNR:"; 
 outString += (PacketSNR);
 outString += " ";
      
 Serial.println("APRS-Ausgangsdaten: ");
 Serial.println(outString);  

// sending APRS sentence via BT
   bluetooth.println(outString); 
   }

///////////////////////////////////////////////////////////////////
void packet_is_OK()
{
  cnt=cnt+1;
  if(cnt>999) { cnt=0; } 
  uint16_t IRQStatus, localCRC;
  IRQStatus = LT.readIrqStatus();                  //read the LoRa device IRQ status register
  RXpacketCount++;
  printElapsedTime();                              //print elapsed time to Serial Monitor
//  LT.printASCIIPacket(RXBUFFER, RXPacketL);        //print the packet as ASCII characters
  Inpacket = (RXBUFFER, RXPacketL);

// Beepton  
//  Serial.print("Beep Volume: ");
//  Serial.println(gucVol);
  if(mrk_x==1)
  {
  M5.update(); 
  ledcSetup( TONE_PIN_CHANNEL, TONE_FREQ, 13);
  SetVolume( gucVol );
  gucToneSts = TONE_PLAY;
  delay(100);
  M5.Speaker.mute();
  gucToneSts = TONE_STOP;
  }
  
// Stringbildung
  byte lora_LLoop;
  byte lora_LData;
  byte lcount;
  
  for (lora_LLoop = 0; lora_LLoop <= RXPacketL; lora_LLoop++)
  {
  lcount++;
  lora_LData = RXBUFFER[lora_LLoop];
  Inpacket += char(lora_LData);     
  } 

// Inpacket = "DJ7OO-15>APRS:!/56m^PTV?yEiQ Lora-APRS.Decoder 433.775 MHz"; 
// Inpacket = "DJ7OO-15>APRS:!4957.13N/00811.97Ek321/035 /A=123456 LoRa-Beacon"; 
     
     int strt = Inpacket.indexOf("<"); 
     Inpacket = Inpacket.substring(strt+3);
     Inpacket2 = Inpacket;

//     Serial.println(" ");      
     Serial.println("LoRa-Eingangsdaten: ");
     Serial.println(Inpacket);
//     Serial.println(" ");      
  
     int rufz = Inpacket.indexOf(">"); 
     Tcall = Inpacket.substring(0,rufz);

//     Serial.print("Call: ");
//     Serial.println(Tcall);

//    ********   SELECT TYPE OF PROTOCOL **********    
     int cnt2 = Inpacket.indexOf("!/"); //compressed
     int cnt2a = Inpacket.indexOf(char(92)); //compressed (BASE91)  
     int cnt2b = Inpacket.indexOf("!L"); //compressed (BASE91)  
     int cnt3 = Inpacket.indexOf(":!"); //uncompressed 
     int cnt4 = Inpacket.indexOf(":="); //uncompressed

     if((cnt2!=-1||cnt2a!=-1||cnt2b!=-1)&&(cnt3!=-1||cnt4!=-1)) 
     {

// **************** Decodierung compressed ****************
      if(cnt2!=-1)
      { aprsdata = Inpacket.substring(cnt2+1,cnt2+14);
        addtext = Inpacket.substring(cnt2+14); }
      else if(cnt2a!=-1)
      { aprsdata = Inpacket.substring(cnt2a,cnt2a+13);
        addtext = Inpacket.substring(cnt2a+13); }
      addtext2 = addtext;

//   Serial.println(aprsdata);
     Serial.print("Text : ");
     Serial.println(addtext);
     
          tab=aprsdata.charAt(0);
     int lat1=aprsdata.charAt(1);
     int lat2=aprsdata.charAt(2);
     int lat3=aprsdata.charAt(3);
     int lat4=aprsdata.charAt(4);
     int lon1=aprsdata.charAt(5);
     int lon2=aprsdata.charAt(6);
     int lon3=aprsdata.charAt(7);
     int lon4=aprsdata.charAt(8);
         symb=aprsdata.charAt(9);
     int alt1=aprsdata.charAt(10);
     int alt2=aprsdata.charAt(11);
     int alt3=aprsdata.charAt(12);
     
     long Lat1 = (lat1-33)*753571; 
     long Lat2 = ((lat2-33)*82810)/10;
     long Lat3 = (lat3-33)*91;
     long Lat4 = lat4-33;
     float Lat5 = Lat1+Lat2+Lat3+Lat4;
     Tlat = 90-(Lat5/380926);

//     Serial.print("Lat:  ");
//     Serial.println(Tlat,5);
     
     long Lon1 = (lon1-33)*753571; 
     long Lon2 = ((lon2-33)*82810)/10;
     long Lon3 = (lon3-33)*91;
     long Lon4 = lon4-33;
     float Lon5 = Lon1+Lon2+Lon3+Lon4;
     Tlon = (Lon5/190463)-180;

//     Serial.print("Lon:  ");
//     Serial.println(Tlon,5);

     if(Tlat<0) { Ns = "S"; } else { Ns = "N"; } 
     if(Tlon<0) { Ew = "W"; } else { Ew = "E"; } 
     
     if(Tlat < 0) { Tlat= -Tlat; }
     unsigned int Deg_Lat = Tlat; 
     Lat = 100*(Deg_Lat) + (Tlat - Deg_Lat)*60; 

     if(Tlon < 0) { Tlon= -Tlon; }
     unsigned int Deg_Lon = Tlon; 
     Lon = 100*(Deg_Lon) + (Tlon - Deg_Lon)*60; 
     
     long Alt1 = (alt1-33)*91;            ; 
     long Alt2 = alt2-33;            ;
     float Alt3 = (Alt1+Alt2);
     Alt3=Alt3*0.00086772215;
     Talt = pow(10,Alt3);
     Talt=Talt/3.281;
     
     Inpacket = "";
     APRS_Mode = 1;
     } 
 
 // *************** Decodierung uncompressed ****************
     else if(cnt2==-1&&(cnt3!=-1||cnt4!=-1)) 
     { 
         if(cnt3!=-1) { APRS = Inpacket.substring(cnt3+2); }
    else if(cnt4!=-1) { APRS = Inpacket.substring(cnt4+2); }

//    Serial.print("APRS: "); Serial.println(APRS);
    
    xLat = APRS.substring(0,7);
    Lat  = xLat.toFloat();
//    Serial.println(Lat);

// APRS LAT-Auswertung  
int xlat11 = xLat.charAt(0);
int xlat12 = xLat.charAt(1);
int xlat13 = xLat.charAt(2);
int xlat14 = xLat.charAt(3);
int xlat15 = xLat.charAt(4);
int xlat16 = xLat.charAt(5);
int xlat17 = xLat.charAt(6);

    int lat_grad2 = (xlat11-48)*10+(xlat12-48);
    float lat_min2 = ((xlat13-48)*1000)+((xlat14-48)*100)+((xlat16-48)*10)+(xlat17-48);

    lat_min2 = lat_min2/6000;
//  Serial.println(lat_grad2+lat_min2,4);
    Tlat =lat_grad2+lat_min2,4;
    
    Ns2 = APRS.substring(7,8);
//    Serial.print(" ");
//    Serial.println(Ns2);

    xLon = APRS.substring(9,17);
    Lon  = xLon.toFloat();
//    Serial.println(Lon);
//    Serial.println(xLon);

// APRS LON-Auswertung  
int xlon11 = xLon.charAt(0);
int xlon12 = xLon.charAt(1);
int xlon13 = xLon.charAt(2);
int xlon14 = xLon.charAt(3);
int xlon15 = xLon.charAt(4);
int xlon17 = xLon.charAt(6);
int xlon18 = xLon.charAt(7);

    int lon_grad2 = ((xlon11-48)*100)+(xlon12-48)*10+(xlon13-48);
    float lon_min2 = ((xlon14-48)*1000)+((xlon15-48)*100)+((xlon17-48)*10)+(xlon18-48);
    lon_min2 = lon_min2/6000;
//  Serial.println(lon_grad2+lon_min2,4);
    Tlon = lon_grad2+lon_min2,4;
    
    Ew2 = APRS.substring(17,18);
//    Serial.print(" ");
//    Serial.println(Ew2);

    tab  = APRS.charAt(8);
    symb = APRS.charAt(18);
    addtext = APRS.substring(19);

//  int pos4a = APRS.indexOf("/");
//  String crs = APRS.substring(pos4a-3,pos3);     // CRS   
//  String spd = APRS_1.substring(pos4a+1,pos3+4); // SPD

    int pos5 = APRS.indexOf("/A=");
    if(pos5!=-1) Alt = APRS.substring(pos5+3,pos5+9);
//  Serial.println(Alt);
    Talt = Alt.toInt(); 
    Talt=Talt/3.281;

    Inpacket = "";
    APRS = "";
    APRS_Mode = 0;

    if(Tlat<0) { Ns = "S"; } else { Ns = "N"; } 
    if(Tlon<0) { Ew = "W"; } else { Ew = "E"; } 
     
    if(Tlat < 0) { Tlat= -Tlat; }
    unsigned int Deg_Lat = Tlat; 
    Lat = 100*(Deg_Lat) + (Tlat - Deg_Lat)*60; 
    if(Tlon < 0) { Tlon= -Tlon; }
    unsigned int Deg_Lon = Tlon; 
    Lon = 100*(Deg_Lon) + (Tlon - Deg_Lon)*60; 

//    Serial.print("Alt:  ");
//    Serial.print(Talt,0);
//    Serial.println("m");

    Serial.print("Text : ");
    Serial.println(addtext);
    Serial.println(" ");
    }

////////////////////////////////////////////////////////////////
// calculate DST/BRG
    
    Lat1c = Lat1a * Deg2Rad;
    Lon1c = Lon1a * Deg2Rad;
    Lat2c = Tlat * Deg2Rad;
    Lon2c = Tlon * Deg2Rad;
    
    Calc_Bearing();
    Calc_Distance();

    Distance2 = Distance*1000;   // calc dist in "m"
    Serial.print("Dst: ");
    Serial.print(Distance2,0);
    Serial.println("m");
    Serial.print("Brg: ");
    Serial.print(Bearing,0);
    Serial.println("°");
    Serial.print("Alt: ");
    Serial.print(Talt,0);
    Serial.println("m");
    
//  extract WX-Data 
    cnt1wx = addtext.indexOf("t");
    cnt2wx = addtext.indexOf("h");
    cnt3wx = addtext.indexOf("b");

//    Serial.print("cnt1wx: ");Serial.println(cnt1wx);
//    Serial.print("cnt2wx: ");Serial.println(cnt2wx);
//    Serial.print("cnt3wx: ");Serial.println(cnt3wx);

// extract batt voltage
    posvolt  = addtext.indexOf("V");
    battvolt = addtext.substring(posvolt-4,posvolt+1);
    Serial.print("UBat:");Serial.println(battvolt);  

    temp = addtext.substring(cnt1wx+1,cnt1wx+4); 
    tempc = temp.toFloat();
    tempc = (tempc-32)*0.555;
    
    humi = addtext.substring(cnt2wx+1,cnt2wx+3); 
    pres = addtext.substring(cnt3wx+1,cnt3wx+5); 
    presc = pres.toFloat();

//    Serial.println(" ");
//    Serial.print("Temp : ");Serial.print(tempc);Serial.println("°C"); 
//    Serial.print("Humi : ");Serial.print(humi);Serial.println("%");
//    Serial.print("Prec : ");Serial.print(pres);Serial.println("hpa");
    Serial.println(" ");
        
    humi2 = humi;
    addtext2 = addtext;

    M5.Lcd.setCursor(260,10);
    if(cnt<100) { M5.Lcd.print(F("0")); }   
    if(cnt<10)  { M5.Lcd.print(F("0")); }
    M5.Lcd.print(cnt);

       if(mrk_cnt==0) { disp_nav(); }
  else if(mrk_cnt==1) { disp_wx(); } 
  else if(mrk_cnt==2) { disp_dstbrg(); }  
  else if(mrk_cnt==3) { disp_raw(); }  

 // send via bluetooth          
    build_aprs(); 
   
    Serial.println(" ");
    temp = "";
    humi = "";
    pres = "";
    xLat = "";
    xLon = "";
    APRS = "";
    addtext = "";
    Inpacket = "";
    outString = "";
}    

////////////////////////////////////////////////////////////////
void led_Flash(uint16_t flashes, uint16_t delaymS)
{
  uint16_t index2;
  for (index2 = 1; index2 <= flashes; index2++)
  {
    digitalWrite(LED1, HIGH);
    delay(delaymS);
    digitalWrite(LED1, LOW);
    delay(delaymS);
  }
}

/////////////////////////////////////////////////////////
 void Button3_ISR() 
{
  State3=digitalRead(button3);
  if(State3==0) { mrk_cnt=mrk_cnt+1; }
  if(mrk_cnt>3) { mrk_cnt=0; }
//  Serial.print("mrk_cnt= ");Serial.println(mrk_cnt);
       if(mrk_cnt==0) { disp_nav(); }
  else if(mrk_cnt==1) { disp_wx(); } 
  else if(mrk_cnt==2) { disp_dstbrg(); }
  else if(mrk_cnt==3) { disp_raw(); }
}

/////////////////////////////////////////////////////////
void Button2_ISR() 
{
  State2=digitalRead(button2);
       if(State2==1&&mrk_y==0) { mrk_y=1; } 
  else if(State2==1&&mrk_y==1) { mrk_y=0; } 
  // ( currently not used )
}

/////////////////////////////////////////////////////////
void Button1_ISR() 
{
  State1=digitalRead(button1);
       if(State1==1&&mrk_x==0) { mrk_x=1; } 
  else if(State1==1&&mrk_x==1) { mrk_x=0; } 
// if(mrk_x==1) { Serial.println("Beep ON"); } 
// else { Serial.println("Beep OFF"); }
} 

///////////////////////////////////////////////////////////
void disp_raw()
{
    M5.Lcd.clear();
    M5.Lcd.setTextSize(3);
    M5.Lcd.setCursor(0,10);
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.println(F("LORA-DECODER"));

    M5.Lcd.setCursor(0, 40);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Rawdata: "));
 
    M5.Lcd.fillRect(0, 69, 320, 3, WHITE);

    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.setCursor(0, 82);
    M5.Lcd.println(Inpacket2);

    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.setCursor(260,10);
    if(cnt<100) {M5.Lcd.print(F("0"));}   
    if(cnt<10)  {M5.Lcd.print(F("0"));}
    M5.Lcd.print(cnt);

    if(mrk_y==1) {
    M5.Lcd.setCursor(300,40);
    M5.Lcd.print("T"); }
}

///////////////////////////////////////////////////////////
void disp_nav()
{
    M5.Lcd.clear();
    M5.Lcd.setTextSize(3);
    M5.Lcd.setCursor(0,10);
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.println(F("LORA-DECODER"));

    M5.Lcd.setCursor(0, 40);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Call: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.println(Tcall);           // Call

    M5.Lcd.fillRect(0, 69, 320, 3, WHITE);

    M5.Lcd.setCursor(0, 82);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Lat : "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.println(Tlat,5);          // Lat

    M5.Lcd.setCursor(0, 112);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Lon : "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.println(Tlon,5);          // Lon

    M5.Lcd.setCursor(0, 142);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("ALTI: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(Talt,0);            // Alti
    M5.Lcd.println(F("m"));

    M5.Lcd.fillRect(0, 175, 320, 3, WHITE);

    M5.Lcd.setTextSize(2);
    M5.Lcd.setCursor(0,187);
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.println(addtext2);                  // addtext
    M5.Lcd.println(" ");

    M5.Lcd.setTextSize(3);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.setCursor(240,142);
    if(APRS_Mode==0) { M5.Lcd.print("APRS"); }
    else { M5.Lcd.print("BASE"); }  
 
    M5.Lcd.setCursor(260,10);
    if(cnt<100) {M5.Lcd.print(F("0"));}   
    if(cnt<10)  {M5.Lcd.print(F("0"));}
    M5.Lcd.print(cnt);
}

///////////////////////////////////////////////////////////
void disp_wx()
{
    M5.Lcd.clear();
    M5.Lcd.setTextSize(3);
    M5.Lcd.setCursor(0,10);
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.println(F("LORA-DECODER"));
     
    M5.Lcd.setCursor(0, 40);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Call: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.println(Tcall);           // Call

    M5.Lcd.fillRect(0, 69, 320, 3, WHITE);

    if(cnt1wx!=-1&&cnt2wx!=-1&&cnt3wx!=-1) 
    {
    M5.Lcd.setCursor(0, 82);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Temp: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(tempc,1);          
    M5.Lcd.println(F(" C"));

    M5.Lcd.setCursor(0, 112);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Humi: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(humi2);          
    M5.Lcd.println(F(" %"));

    M5.Lcd.setCursor(0, 142);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Pres: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(presc,0);          
    M5.Lcd.println(F(" hPa"));

    M5.Lcd.fillRect(0, 175, 320, 3, WHITE);
    
    M5.Lcd.setCursor(0, 188);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("RSSI: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(PacketRSSI);                 // RSSI
    M5.Lcd.println(F(" dBm"));

    M5.Lcd.setCursor(0, 216); 
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("SNR : "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(PacketSNR);                  // SNR
    M5.Lcd.println(F(" dB")); 

    M5.Lcd.setCursor(220, 216);               // VBat     
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.println(battvolt);
    }
    else
    {
    M5.Lcd.setCursor(60, 100); 
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("NO WX DATA"));
    }  
    
    M5.Lcd.setCursor(260,10);
    if(cnt<100) {M5.Lcd.print(F("0"));}   
    if(cnt<10)  {M5.Lcd.print(F("0"));}
    M5.Lcd.print(cnt);
}

///////////////////////////////////////////////////////////
void disp_dstbrg()
{
    M5.Lcd.clear();
    M5.Lcd.setTextSize(3);
    M5.Lcd.setCursor(0,10);
    M5.Lcd.setTextColor(GREEN);
    M5.Lcd.println(F("LORA-DECODER"));
      
    M5.Lcd.setCursor(0, 40);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("Call: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.println(Tcall);           // Call

    M5.Lcd.fillRect(0, 69, 320, 3, WHITE);
    
    M5.Lcd.setCursor(0, 82); 
    M5.Lcd.print(F("DST : "));       // Dst
    if(Distance2>999) 
    { float Distance3=Distance2/1000; 
    M5.Lcd.print(Distance3,1);
    M5.Lcd.println(" Km"); 
    }
    else
    { 
    M5.Lcd.print(Distance2,0);
    M5.Lcd.println(" m"); 
    }

    M5.Lcd.setCursor(0, 112);        // Brg
    M5.Lcd.print(F("BRG : "));
    M5.Lcd.print(Bearing,0);
    M5.Lcd.println(" degs.");      

    M5.Lcd.setCursor(0, 142); 
    M5.Lcd.print(F("ALTI: "));
    M5.Lcd.print(Talt,0);            // Alti
    M5.Lcd.println(" m");

    M5.Lcd.fillRect(0, 175, 320, 3, WHITE);
    
    M5.Lcd.setCursor(0, 188);
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("RSSI: "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(PacketRSSI);                 // RSSI
    M5.Lcd.println(F(" dBm"));            

    M5.Lcd.setCursor(0, 216); 
    M5.Lcd.setTextColor(WHITE);
    M5.Lcd.print(F("SNR : "));
    M5.Lcd.setTextColor(YELLOW);
    M5.Lcd.print(PacketSNR);                  // SNR
    M5.Lcd.println(F(" dB"));

    M5.Lcd.setCursor(260,10);
    if(cnt<100) {M5.Lcd.print(F("0"));}   
    if(cnt<10)  {M5.Lcd.print(F("0"));}
    M5.Lcd.print(cnt);
}

//////////////////////////////////////////////////////////////////
void setup()
{
  Serial.begin(9600);
  Serial.println();

  M5.begin();
  M5.Lcd.clear();
  M5.Lcd.setCursor(0, 10);
  M5.Lcd.setTextColor(GREEN);
  M5.Lcd.setTextSize(3);
  M5.Lcd.println(F("LORA-APRS-DECODER"));
  M5.Lcd.setTextColor(WHITE);
  M5.Lcd.setCursor(0, 40);
  M5.Lcd.println(F("   433.775 MHz"));
  M5.Lcd.setCursor(0, 70);
  M5.Lcd.println(F("  BT-OUT:  APRS"));
  M5.Lcd.setCursor(0, 140);
  M5.Lcd.setTextColor(YELLOW);
  M5.Lcd.println(F("  WAIT FOR DATA !"));
  M5.Lcd.setTextColor(WHITE);
  M5.Lcd.setCursor(0, 200);
  M5.Lcd.println(F("  (C)DJ7OO 2020"));

  bluetooth.begin("M5S_2_LoRa_APRS");
  M5.Speaker.begin();

  pinMode(button1, INPUT);
  pinMode(button2, INPUT);
  pinMode(button3, INPUT);
  
  attachInterrupt(digitalPinToInterrupt(button1), Button1_ISR, CHANGE); 
  attachInterrupt(digitalPinToInterrupt(button2), Button2_ISR, CHANGE);  
  attachInterrupt(digitalPinToInterrupt(button3), Button3_ISR, CHANGE); 

  pinMode(LED1, OUTPUT);                        //setup pin as output for indicator LED
  led_Flash(2, 125);                            //two quick LED flashes to indicate program start

  if (VCCPOWER >= 0)
  {
    pinMode(VCCPOWER, OUTPUT);                  //For controlling power to external devices
    digitalWrite(VCCPOWER, LOW);                //VCCOUT on. lora device on
  }
  Serial.begin(9600);
  delay(2000);
  Serial.println();
  Serial.print(F(__TIME__));
  Serial.print(F(" "));
  Serial.println(F(__DATE__));
  Serial.println(F(Program_Version));
  Serial.println();
  Serial.println(F("LoRa_Receiver_ESP32"));
  Serial.println();
  if (BUZZER > 0)
  {
    pinMode(BUZZER, OUTPUT);
    digitalWrite(BUZZER, HIGH);
    delay(50);
    digitalWrite(BUZZER, LOW);
  }
 //SPI.begin();                                      //default setup can be used be used
  SPI.begin(SCK, MISO, MOSI, NSS);                   //alternative format for SPI3, VSPI; SPI.begin(SCK,MISO,MOSI,SS)

 //SPI beginTranscation is normally part of library routines, but if it is disabled in the library
 //a single instance is needed here, so uncomment the program line below
 //SPI.beginTransaction(SPISettings(8000000, MSBFIRST, SPI_MODE0));
 //setup hardware pins used by device, then check if device is found
  if (LT.begin(NSS, NRESET, DIO0, DIO1, DIO2, LORA_DEVICE))
  {
    Serial.println(F("LoRa Device found"));
    led_Flash(2, 125);
    delay(1000);
  }
  else
  {
    Serial.println(F("No device responding"));
    while (1)
    {
      led_Flash(50, 50);                                       //long fast speed LED flash indicates device error
    }
  }
 
 //The function call list below shows the complete setup for the LoRa device using the information defined in the
 //Settings.h file.
 //The 'Setup LoRa device' list below can be replaced with a single function call;
 //LT.setupLoRa(Frequency, Offset, SpreadingFactor, Bandwidth, CodeRate, Optimisation);

 //***************************************************************************************************
 //Setup LoRa device
 //***************************************************************************************************
  LT.setMode(MODE_STDBY_RC);                              //got to standby mode to configure device
  LT.setPacketType(PACKET_TYPE_LORA);                     //set for LoRa transmissions
  LT.setRfFrequency(Frequency, Offset);                   //set the operating frequency
  LT.calibrateImage(0);                                   //run calibration after setting frequency
  LT.setModulationParams(SpreadingFactor, Bandwidth, CodeRate, LDRO_AUTO);  //set LoRa modem parameters
  LT.setBufferBaseAddress(0x00, 0x00);                    //where in the SX buffer packets start, TX and RX
  LT.setPacketParams(8, LORA_PACKET_VARIABLE_LENGTH, 255, LORA_CRC_ON, LORA_IQ_NORMAL);  //set packet parameters
  LT.setSyncWord(LORA_MAC_PRIVATE_SYNCWORD);              //syncword, LORA_MAC_PRIVATE_SYNCWORD = 0x12, or LORA_MAC_PUBLIC_SYNCWORD = 0x34
  LT.setHighSensitivity();                                //set for highest sensitivity at expense of slightly higher LNA current
  LT.setDioIrqParams(IRQ_RADIO_ALL, IRQ_RX_DONE, 0, 0);   //set for IRQ on RX done
 
 //***************************************************************************************************
  Serial.println();
  LT.printModemSettings();                                     //reads and prints the configured LoRa settings, useful check
  Serial.println();
  LT.printOperatingSettings();                                 //reads and prints the configured operting settings, useful check
//  Serial.println();
//  Serial.println();
//  LT.printRegisters(0x00, 0x4F);                               //print contents of device registers, normally 0x00 to 0x4F
//  Serial.println();
  Serial.println(" ");
  Serial.print(F("RECEIVER READY"));
  Serial.println(" ");
}
